#! /bin/bash
# Additional library for authors of exercises for FW4EX.
# This library should be sourced.
source $FW4EX_LIB_DIR/basicLib.sh
source $FW4EX_LIB_DIR/moreLib.sh
source $FW4EX_LIB_DIR/Patterns/comparisonLib.sh

COMMAND=reparation
WIN_FORMULA='triangular 0 0 1 0.88/$DATA_NUMBER'
#DEBUG=true

# Dans quel fichier se trouve la solution:
SOLUTION=${SOLUTION:-$FW4EX_EXERCISE_DIR/pseudos/perfect/$COMMAND}

# Ou sont les repertoires 
DATA_DIR=${DATA_DIR:-$FW4EX_EXERCISE_DIR/tests}
DATA_SUFFIX=${DATA_SUFFIX:-d}

# Comment normaliser les resultats avant comparaison.
# Ne pas definir NORMALIZER si l'on n'en a pas besoin.

# Par defaut on utilise une distance de Levenshtein. On peut aussi
# utiliser une comparaison par ligne (constante 'line'):
COMPARISON=${COMPARISON:-char}

# Pour mettre certains details en commentaires html
DEBUG=${DEBUG:-false}

# On pourra aussi dans WIN_FORMULA utiliser la variable DATA_NUMBER

# -1- ###################################################################
# Retour vers l'etudiant de son script:
fw4ex_show_script "$COMMAND"
# NOTE: il y a un defaut de transmission des droits dans le serveur A.
chmod a+x "$COMMAND"

regenerate_and_goto_test_directory () {
    local ORGDIR=$1
    local D=$2
    cd $TMPDIR
    rm -rf $D 2>/dev/null
    mkdir $D
    rsync -au $ORGDIR/. $D/
    cd $D
}

# -3- ###################################################################
# Executer et comparer
# le nombre de repertoires a tester. Chaque test rapportera
# 1/$DATA_NUMBER point (c'est specifie dans WIN_FORMULA).
DATA_NUMBER=$( ls -d1 $DATA_DIR/*.${DATA_SUFFIX}/ | wc -l )
cat <<EOF
<p>
Je vais comparer votre solution et la mienne sur $DATA_NUMBER repertoires.
</p>
EOF

# Le repertoire qui sera utilise pour les tests:
D=$( mktemp -d -p $TMPDIR )
COMMAND=$(pwd)/$COMMAND

I=0
# On itere sur toutes les configurations de tests:
for data in $DATA_DIR/*.${DATA_SUFFIX}/
do 
    I=$(( $I+1 ))
    (

    # -3a- Decrire la configuration de test
        regenerate_and_goto_test_directory $data $D
        echo "<p>Voici le contenu du répertoire à réparer: <pre>"
        ls | fw4ex_transcode_carefully
        echo "</pre></p>"

    # -3b- tourner la commande de l'etudiant
        fw4ex_run_student_command "$COMMAND"
        if [[ -s $TMPDIR/results.s ]]
        then
            fw4ex_show_student_raw_result
        else
            echo "<p>Votre commande n'a rien produit sur son flux de 
sortie. </p>"
        fi
        # Pour cette question, on penalise les anomalies:
        if [[ ! -s $TMPDIR/err.s ]]
        then 
            echo "<p>J'observe que vous n'avez produit aucune erreur. Vous
gagnez "
            fw4ex_win 0.12/$DATA_NUMBER
            echo "points.</p>"
        fi
        echo "<p>Voici le contenu final du répertoire que vous avez 
réparé: <pre>"
        ls | tee $TMPDIR/result.s | fw4ex_transcode_carefully
        echo "</pre></p>"

    # -3c- tourner la commande de l'auteur
    # Si la solution de l'auteur provoquent des anomalies ou si le
    # normaliseur en provoque, les erreurs sont emises sur le stderr
    # qui, non vide, sera signale a l'auteur.

        regenerate_and_goto_test_directory $data $D

        fw4ex_run_teacher_command "$SOLUTION"
        if [[ -s $TMPDIR/result.t ]]
        then
            fw4ex_show_teacher_raw_result
        else
            echo "<p>Ma commande n'a rien produit sur son flux de 
sortie. </p>"
        fi
        echo "<p>Voici le contenu final du répertoire que j'ai réparé: <pre>"
        ls | tee $TMPDIR/result.t | fw4ex_transcode_carefully
        echo "</pre></p>"

    # -3e- comparer et donner des points
        case "${COMPARISON}" in 
            char)
                fw4ex_compare_results_as_strings
                ;;
            line)
                fw4ex_compare_results_as_lines
                ;;
            *)
                echo "Cannot find such a comparison ($COMPARISON) ?" 1>&2
                exit 101
                ;;
        esac
    )
done

cat <<EOF
<p>
Fin de correction automatique.
</p>
EOF

# Local Variables: ##
# coding:utf-8 ##
# End: ##
# end of check.sh
